local MINOR_VERSION = tonumber(("$Revision: 66572 $"):match("%d+"))
local PlayerName = UnitName("player")
local Threat = LibStub("Threat-2.0")
local math_floor = math.floor
local timers = {}
local AceConfig = LibStub("AceConfigDialog-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Omen")

Omen = LibStub("AceAddon-3.0"):NewAddon("Omen", "AceEvent-3.0", "AceTimer-3.0", "AceBucket-3.0", "AceConsole-3.0", "LibSink-2.0")
Omen.MINOR_VERSION = MINOR_VERSION

-- Silently fail embedding if it doesn't exist
LibStub("AceAddon-3.0"):EmbedLibrary(Omen, "LibFuBarPlugin-Mod-3.0", true)

Omen:SetDefaultModuleState(false)
Omen.SkinList = {}

local optFrame
function Omen:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("Omen2DB")
	self.db:RegisterDefaults({
		global = {
			skins = {}
		},
		profile = {
			activeSkin = {},
			options = {},
			sinkOptions = {}
		}
	})
	self:SetSinkStorage(self.db.profile.sinkOptions)
	self.configOptions.args.output = self:GetSinkAce3OptionsDataTable()
	self:CreateFrame()
	if self.Anchor:GetHeight() < 120 then
		self.Anchor:SetHeight(120)
	end
	local active = Omen:GetOption("ActiveWhenSolo")
	if active then
		Threat:RequestActiveOnSolo(active)
	end
	Omen:PopulateSkinList()

	if LibStub:GetLibrary("LibFuBarPlugin-Mod-3.0", true) then
		-- Create the FuBarPlugin bits.
		self:SetFuBarOption("GameTooltip", true)
		self:SetFuBarOption("hasNoColor", true)
		self:SetFuBarOption("cannotDetachTooltip", true)
		self:SetFuBarOption("hideWithoutStandby", true)
		self:SetFuBarOption("cannotDetachTooltip", true)
		self:SetFuBarOption("iconPath", [[Interface\AddOns\Omen\icon]])	
	end
		
	if self:GetOption("Standby") then
		self:SetEnabledState(false)
	end
	
	self.version = ("Omen r%s / Threat-2.0 r%s"):format(Omen.MINOR_VERSION, LibStub.minors["Threat-2.0"])
	
	optFrame = AceConfig:AddToBlizOptions("Omen", "Omen")
	
	self.configOptions.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
end

function Omen:OnUpdateFuBarTooltip()
	GameTooltip:AddLine("|cffffff00" .. L["Click|r to toggle the Omen window"])
	GameTooltip:AddLine("|cffffff00" .. L["Right-click|r to open the options menu"])
end

function Omen:OnFuBarClick(button)
	self:Toggle()
end

function Omen:OnEnable()
	self:LayoutModuleIcons()
	local module = self:GetOption("Modules.LastActive")
	if module then
		-- self:DisableModule(module)
		self:EnableModule(module)
	else
		for k, v in self:IterateModules() do
			-- v:Disable()
			v:Enable()
			break
		end
	end
	self:Toggle(self:GetOption("Show"))
	
	Threat.RegisterCallback(self, "Activate", "UpdateVisible")
	Threat.RegisterCallback(self, "Deactivate", "UpdateVisible")
	
	if self:GetOption("FuBar.HideMinimapButton", false) then
		self:Hide()
	end
end

function Omen:OnDisable()
	Threat.UnregisterCallback(self, "Activate")
	Threat.UnregisterCallback(self, "Deactivate")
	self.Anchor:Hide()
end

function Omen:Test()
	self = Omen
	if self.activeModule then
		self.activeModule:Test()
	end
end

function Omen:Flash(self, warningLevel)
	UIFrameFlash(LowHealthFrame, 0.15, 0.75, 2.6, false, 0, 0.4);
end

function Omen:Toggle(setting)
	if setting ~= nil then
		self:SetOption("Show", setting)
	else
		self:SetOption("Show", not Omen.Anchor:IsVisible())
	end
	if self:GetOption("Show") then
		self.Anchor:Show()
	else
		self.Anchor:Hide()
	end
end

--------------------------------
-- Yoinked from BigWigs
--------------------------------

-- Shake properties
local shaking = nil
local SHAKE_DURATION = 0.8
local SHAKE_X = 10
local SHAKE_Y = 10
local fail = nil
local shaker = nil

local originalPoints = nil
local function startShake()
	if not shaking then
		-- store old worldframe positions, we need them all, people have frame modifiers for it etc.
		if not originalPoints then
			originalPoints = {}
			for i = 1, WorldFrame:GetNumPoints() do
				table.insert(originalPoints, {WorldFrame:GetPoint(i)})
			end
		end
		shaking = SHAKE_DURATION -- don't think we want to make this a setting.
		shaker:Show()
	end
end

local function shakeOnUpdate(frame, elapsed)
	shaking = shaking - elapsed
	local x, y = 0, 0 -- Resets to original position if we're supposed to stop.
	if shaking <= 0 then -- stop shaking
		shaking = nil
		shaker:Hide()
	else
		x = math.random(-SHAKE_X,SHAKE_X)
		y = math.random(-SHAKE_Y,SHAKE_Y)
	end
	WorldFrame:ClearAllPoints()
	for i, v in ipairs(originalPoints) do
		WorldFrame:SetPoint(v[1], v[2], v[3], v[4] + x, v[5] + y)
	end
end

--------------------------------
-- End yoink
--------------------------------

function Omen:Shake()
	if not shaker then
		shaker = CreateFrame("Frame", "OmenShaker", UIParent)
		shaker:Hide()
		shaker:SetScript("OnUpdate", shakeOnUpdate)
	end
	local check = WorldFrame:IsProtected()
	if not check then
		startShake()
	elseif check and not fail then
		Omen:Print(("|cffff0000%s:|r %s"):format(L["Error"], L["Unable to use shake warning. If you have nameplates turned on, turn them off to enable shake warnings."]))
		fail = true
	end
end

function Omen:Warn(sound, flash, shake, message)
	if sound then
		PlaySoundFile(sound)
	end
	if flash then self:Flash() end
	if shake then self:Shake() end
	if message then self:Pour(message, 1, 0, 0, nil, 24, "OUTLINE", true) end
end

local setupUIFrame = false
function Omen:ShowConfig()
	if IsModifierKeyDown() then
		InterfaceOptionsFrame:SetWidth(700)
		if not setupUIFrame then
			InterfaceOptionsFrame:SetMovable(true)
			InterfaceOptionsFrame:SetScript("OnMouseDown", function() this:StartMoving() end)
			InterfaceOptionsFrame:SetScript("OnMouseUp", function() this:StopMovingOrSizing() end)
			setupUIFrame = true
		end
		InterfaceOptionsFrame_OpenToFrame(optFrame)
	else
		AceConfig:SetDefaultSize("Omen", 500, 550)
		AceConfig:Open("Omen", configFrame)
	end
end
Omen.OpenMenu = Omen.ShowConfig		-- for fubar

function Omen:ShowDropdown()
	local dd = LibStub("LibDropdown-1.0")
	if dd then
		dd:OpenAce3Menu(self.configOptions)
	end
end
